==> truffle-nfi-panama-23.1.2-sources.jar: com/oracle/truffle/nfi/backend/panama/ClosureArgumentNode.java <==
/*
 * Copyright (c) 2017, 2021, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.nfi.backend.panama;

import com.oracle.truffle.api.dsl.Fallback;
import com.oracle.truffle.api.dsl.NodeChild;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.api.nodes.Node;

import java.lang.foreign.MemorySegment;

abstract class ClosureArgumentNode extends Node {

    public abstract Object execute(VirtualFrame frame);

    static final class ConstArgumentNode extends ClosureArgumentNode {

        private final Object value;

        ConstArgumentNode(Object value) {
            this.value = value;
        }

        @Override
        public Object execute(VirtualFrame frame) {
            return value;
        }
    }

    static final class GetArgumentNode extends ClosureArgumentNode {

        private final int index;

        GetArgumentNode(int index) {
            this.index = index;
        }

        @Override
        public Object execute(VirtualFrame frame) {
            Object[] args = (Object[]) frame.getArguments()[1];
            if (index == args.length) {
                return frame.getArguments()[0];
            }
            return args[index];
        }
    }

    @NodeChild(value = "argument", type = ClosureArgumentNode.class)
    abstract static class GenericClosureArgumentNode extends ClosureArgumentNode {
        final PanamaType type;

        GenericClosureArgumentNode(PanamaType type) {
            this.type = type;
        }

        @Specialization(guards = "arg == null")
        Object doNull(@SuppressWarnings("unused") Object arg) {
            return NativePointer.NULL;
        }

        @Fallback
        Object doGeneric(Object arg) {
            return arg;
        }
    }

    @NodeChild(value = "argument", type = ClosureArgumentNode.class)
    abstract static class StringClosureArgumentNode extends ClosureArgumentNode {
        final PanamaType type;

        StringClosureArgumentNode(PanamaType type) {
            this.type = type;
        }

        @Specialization(guards = "arg == null")
        Object doNull(@SuppressWarnings("unused") Object arg) {
            return new NativeString(0);
        }

        @Fallback
        @SuppressWarnings("preview")
        Object doString(Object arg) {
            return ((MemorySegment) arg).getUtf8String(0);
        }
    }
}

==> truffle-nfi-panama-24.0.0-sources.jar: com/oracle/truffle/nfi/backend/panama/ClosureArgumentNode.java <==
/*
 * Copyright (c) 2017, 2021, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.nfi.backend.panama;

import java.lang.foreign.MemorySegment;

import com.oracle.truffle.api.dsl.Fallback;
import com.oracle.truffle.api.dsl.NodeChild;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.api.nodes.Node;

abstract class ClosureArgumentNode extends Node {

    public abstract Object execute(VirtualFrame frame);

    static final class ConstArgumentNode extends ClosureArgumentNode {

        private final Object value;

        ConstArgumentNode(Object value) {
            this.value = value;
        }

        @Override
        public Object execute(VirtualFrame frame) {
            return value;
        }
    }

    static final class GetArgumentNode extends ClosureArgumentNode {

        private final int index;

        GetArgumentNode(int index) {
            this.index = index;
        }

        @Override
        public Object execute(VirtualFrame frame) {
            Object[] args = (Object[]) frame.getArguments()[1];
            if (index == args.length) {
                return frame.getArguments()[0];
            }
            return args[index];
        }
    }

    @NodeChild(value = "argument", type = ClosureArgumentNode.class)
    abstract static class GenericClosureArgumentNode extends ClosureArgumentNode {
        final PanamaType type;

        GenericClosureArgumentNode(PanamaType type) {
            this.type = type;
        }

        @Specialization(guards = "arg == null")
        Object doNull(@SuppressWarnings("unused") Object arg) {
            return NativePointer.NULL;
        }

        @Fallback
        Object doGeneric(Object arg) {
            return arg;
        }
    }

    @NodeChild(value = "argument", type = ClosureArgumentNode.class)
    abstract static class StringClosureArgumentNode extends ClosureArgumentNode {
        final PanamaType type;

        StringClosureArgumentNode(PanamaType type) {
            this.type = type;
        }

        @Specialization(guards = "arg == null")
        Object doNull(@SuppressWarnings("unused") Object arg) {
            return new NativeString(0);
        }

        @Fallback
        Object doString(Object arg) {
            return ((MemorySegment) arg).getString(0);
        }
    }
}
